<?php
/**
 * Plugin Name: CeeVee plugin
 * Description: Custom post types for CeeVee wp theme 
 * Version: 1.0.0
 * Author: Apollo13
 * Author URI: http://apollo13.eu/
 * Text Domain: ceevee-plugin
 * License: GPL2

 */

/**
 * Get some constants ready for paths when your plugin grows 
 * 
 */

define( 'CEEVEE_PLUGIN_VERSION', '1.0.0' );
define( 'CEEVEE_PLUGIN_PATH', dirname( __FILE__ ) );
define( 'CEEVEE_PLUGIN_IMG_PATH', CEEVEE_PLUGIN_PATH . '/img/' );
define( 'CEEVEE_PLUGIN_FOLDER', basename( CEEVEE_PLUGIN_PATH ) );
define( 'CEEVEE_PLUGIN_URL', plugins_url() . '/' . CEEVEE_PLUGIN_FOLDER );

define('A13_CUSTOM_POST_TYPE_WORK', 'work');
define('A13_CUSTOM_POST_TYPE_WORK_SLUG', 'work'); //defined a bit lower
define('A13_CUSTOM_POST_TYPE_NAV_A', 'nava');
define('A13_CUSTOM_POST_TYPE_PEOPLE', 'people');
define('A13_CPT_WORK_TAXONOMY', 'genre');
define('A13_CPT_PEOPLE_TAXONOMY', 'group');


/**
 * 
 * The plugin base class - the root of all WP goods!
 * 
 * @author nofearinc
 *
 */
class CeeVee_Plugin_Base {
	
	/**
	 * 
	 * Assign everything as a call from within the constructor
	 */
	public function __construct() {
		// add script and style calls the WP way 
		// it's a bit confusing as styles are called with a scripts hook
		// @blamenacin - http://make.wordpress.org/core/2011/12/12/use-wp_enqueue_scripts-not-wp_print_styles-to-enqueue-scripts-and-styles-for-the-frontend/
		add_action( 'wp_enqueue_scripts', array( $this, 'ceevee_add_JS' ) );
		add_action( 'wp_enqueue_scripts', array( $this, 'ceevee_add_CSS' ) );
		
		// Register custom post types and taxonomies
		add_action( 'init', array( $this, 'ceevee_custom_post_types_callback' ), 5 );
		
		// Translation-ready
		add_action( 'plugins_loaded', array( $this, 'ceevee_add_textdomain' ) );
		
		
	}	
	
	/**
	 * 
	 * Adding JavaScript scripts
	 * 
	 * Loading existing scripts from wp-includes or adding custom ones
	 * 
	 */
	public function ceevee_add_JS() {
		wp_enqueue_script( 'jquery' );
		// load custom JSes and put them in footer
		wp_register_script( 'ceevee_plugin-script', plugins_url( '/script.js' , __FILE__ ), array('jquery'), CEEVEE_PLUGIN_VERSION, true );
		wp_enqueue_script( 'ceevee_plugin-script' );
	}
	
	
	
	/**
	 * 
	 * Add CSS styles
	 * 
	 */
	public function ceevee_add_CSS() {
		wp_register_style( 'ceevee_plugin-style', plugins_url( '/style.css', __FILE__ ), array(), CEEVEE_PLUGIN_VERSION, 'screen' );
		wp_enqueue_style( 'ceevee_plugin-style' );
	}
	
	
	

	
	/**
	 * Register custom post types
     *
	 */
	public function ceevee_custom_post_types_callback() {
		
        //nava
        $labels = $supports = $args = $type_labels = '';
        $labels = array(
            'name' =>           __( 'Navigation anchors', 'ceevee-plugin' ),
            'singular_name' =>  __( 'Navigation anchor', 'ceevee-plugin' ),
            'add_new' =>        __( 'Add New', 'ceevee-plugin' ),
            'add_new_item' =>   __( 'Add New', 'ceevee-plugin' ),
            'edit_item' =>      __( 'Edit', 'ceevee-plugin' ),
            'new_item' =>       __( 'New', 'ceevee-plugin' ),
            'view_item' =>      __( 'View', 'ceevee-plugin' ),
            'search_items' =>   __( 'Search', 'ceevee-plugin' ),
            'not_found' =>      __( 'Nothing found', 'ceevee-plugin' ),
            'not_found_in_trash' => __( 'Nothing found in Trash', 'ceevee-plugin' ),
            'parent_item_colon' => ''
        );

        $supports = array(  );
        

        $args = array(
            'labels' => $labels,
            'exclude_from_search' => true,
            'public' => true,
            'show_in_menu' => true,
            'show_in_nav_menus' => true,
            'menu_position' => 5,
            'publicly_queryable' => true,
            'query_var' => true,
            'rewrite' => false,
            'supports' => $supports,
        );
        
        register_post_type( A13_CUSTOM_POST_TYPE_NAV_A , $args );
        
        //people
        $labels = $supports = $args = $type_labels = '';
        $labels = array(
            'name' =>           __( 'People', 'ceevee-plugin' ),
            'singular_name' =>  __( 'Man', 'ceevee-plugin' ),
            'add_new' =>        __( 'Add New', 'ceevee-plugin' ),
            'add_new_item' =>   __( 'Add New', 'ceevee-plugin' ),
            'edit_item' =>      __( 'Edit', 'ceevee-plugin' ),
            'new_item' =>       __( 'New', 'ceevee-plugin' ),
            'view_item' =>      __( 'View', 'ceevee-plugin' ),
            'search_items' =>   __( 'Search', 'ceevee-plugin' ),
            'not_found' =>      __( 'Nothing found', 'ceevee-plugin' ),
            'not_found_in_trash' => __( 'Nothing found in Trash', 'ceevee-plugin' ),
            'parent_item_colon' => ''
        );

        $supports = array( 'title','thumbnail','editor','page-attributes' );
        
        $args = array(
            'labels' => $labels,
            'exclude_from_search' => true,
            'public' => true,
            'show_in_menu' => true,
            'show_in_nav_menus' => false,
            'menu_position' => 6,
            'publicly_queryable' => true,
            'query_var' => true,
            'rewrite' => false,
            'supports' => $supports,
        );

        register_post_type( A13_CUSTOM_POST_TYPE_PEOPLE , $args );
        
        $group_labels = array(
            'name' => __( 'People groups', 'ceevee-plugin' ),
            'singular_name' => __( 'Group', 'ceevee-plugin' ),
            'search_items' =>  __( 'Search Groups', 'ceevee-plugin' ),
            'popular_items' =>  __( 'Popular Groups', 'ceevee-plugin' ),
            'all_items' => __( 'All Groups', 'ceevee-plugin' ),
            'parent_item' => __( 'Parent Group', 'ceevee-plugin' ),
            'parent_item_colon' => __( 'Parent Group:', 'ceevee-plugin' ),
            'edit_item' => __( 'Edit Group', 'ceevee-plugin' ),
            'update_item' => __( 'Update Group', 'ceevee-plugin' ),
            'add_new_item' => __( 'Add New Group', 'ceevee-plugin' ),
            'new_item_name' => __( 'New Group Name', 'ceevee-plugin' ),
            'menu_name' => __( 'Group', 'ceevee-plugin' ),
        );

        register_taxonomy(A13_CPT_PEOPLE_TAXONOMY, array(A13_CUSTOM_POST_TYPE_PEOPLE),
            array(
                "hierarchical" => false,
                "label" => __('People groups', 'ceevee-plugin'),
                "labels" => $group_labels,
                "rewrite" => true,
                'show_admin_column' => true
            )
        );
        
        //work
        $labels = $supports = $args = $genre_labels = '';
        $labels = array(
            'name' =>           __( 'My Works', 'ceevee-plugin' ),
            'singular_name' =>  __( 'Work', 'ceevee-plugin' ),
            'add_new' =>        __( 'Add New', 'ceevee-plugin' ),
            'add_new_item' =>   __( 'Add New Work', 'ceevee-plugin' ),
            'edit_item' =>      __( 'Edit Work', 'ceevee-plugin' ),
            'new_item' =>       __( 'New Work', 'ceevee-plugin' ),
            'view_item' =>      __( 'View Work', 'ceevee-plugin' ),
            'search_items' =>   __( 'Search Works', 'ceevee-plugin' ),
            'not_found' =>      __( 'Nothing found', 'ceevee-plugin' ),
            'not_found_in_trash' => __( 'Nothing found in Trash', 'ceevee-plugin' ),
            'parent_item_colon' => ''
        );

        $supports = array( 'title','thumbnail','editor','excerpt' );
       

        $args = array(
            'labels' => $labels,
            'public' => true,
            'query_var' => true,
            'menu_position' => 5,
            'rewrite' =>  array('slug' => A13_CUSTOM_POST_TYPE_WORK_SLUG),
            'supports' => $supports,
        );

        register_post_type( A13_CUSTOM_POST_TYPE_WORK , $args );

        $genre_labels = array(
            'name' => __( 'Works Genres', 'ceevee-plugin' ),
            'singular_name' => __( 'Genre', 'ceevee-plugin' ),
            'search_items' =>  __( 'Search Genres', 'ceevee-plugin' ),
            'popular_items' =>  __( 'Popular Genres', 'ceevee-plugin' ),
            'all_items' => __( 'All Genres', 'ceevee-plugin' ),
            'parent_item' => __( 'Parent Genre', 'ceevee-plugin' ),
            'parent_item_colon' => __( 'Parent Genre:', 'ceevee-plugin' ),
            'edit_item' => __( 'Edit Genre', 'ceevee-plugin' ),
            'update_item' => __( 'Update Genre', 'ceevee-plugin' ),
            'add_new_item' => __( 'Add New Genre', 'ceevee-plugin' ),
            'new_item_name' => __( 'New Genre Name', 'ceevee-plugin' ),
            'menu_name' => __( 'Genre', 'ceevee-plugin' ),
        );

        register_taxonomy(A13_CPT_WORK_TAXONOMY, array(A13_CUSTOM_POST_TYPE_WORK),
            array(
                "hierarchical" => false,
                "label" => __('Works Genres', 'ceevee-plugin'),
                "labels" => $genre_labels,
                "rewrite" => true,
                'show_admin_column' => true
            )
        );
        
        
	}
	
	
	/**
	 * Add textdomain for plugin
	 */
	public function ceevee_add_textdomain() {
		load_plugin_textdomain( 'ceevee-plugin', false, dirname( plugin_basename( __FILE__ ) ) . '/lang/' );
	}
	
	
}

/*
 * Making cover for galleries in Galleries list
 */
if(!function_exists('a13_make_gallery_image')){
    function a13_make_gallery_image( $gallery_id, $thumb_size, $hidden = false ){
        if(empty($gallery_id)){
            $gallery_id = get_the_ID();
        }
        if ( has_post_thumbnail($gallery_id) ) {
            $src = wp_get_attachment_image_src( get_post_thumbnail_id ( $gallery_id ), $thumb_size );
            $src = $src[0];
        }
        else{
            $src = CEEVEE_PLUGIN_IMG_PATH . '/holders/photo.jpg';
        }

        $style = '';
        if($hidden){
            $style = ' style="visibility: hidden;"';
        }

        return '<img src="'.esc_url($src).'" alt=""'.$style.' />';
    }
}

//nava functions
function apollo13_after_page_save( $post_id ) {

	// If this is just a revision - exit
	if ( wp_is_post_revision( $post_id ) ){ 
    	return;
    }
    // avoid generating nava in case of page import
    if( isset($_SESSION['import_is_runnig']) && $_SESSION['import_is_runnig'] == 1){
        return;
    } 
	$page = get_post( $post_id );
    
    $a13_nava_page_slug = $page->post_name;
    $a13_nava_posts = array();
    //prepare array of params with a13_nava = 1
    //search for vc_row shortcodes inside page
    $pattern = get_shortcode_regex();
    preg_match_all( '/\[(\[?)(vc_row)(?![\w-])([^\]\/]*(?:\/(?!\])[^\]\/]*)*?)(?:(\/)\]|\](?:([^\[]*+(?:\[(?!\/\2\])[^\[]*+)*+)\[\/\2\])?)(\]?)/s', $page->post_content, $matches );
    // array of shortcode's params
    $param_sets = $matches[3];
    
    if( empty($param_sets) ){

        //return;
    } 
    
    foreach ( $param_sets as $param_set ){
        if ( stripos( $param_set, 'a13_nava="1"' ) === false ){
            continue;
        }
        $found = false;
        $a13_nava_title = '';
        //get shortcode's params
        $params = array();
        
        $params = explode( '" ', $param_set );
        foreach ( $params as $param ){
            $parts = array();
            
            $parts = explode( '=', $param );
            if ( $parts[0] == 'a13_nava_title' ){
                $a13_nava_title = str_replace('"','',$parts[1]);
                $found = true;
            }
        }
        
        if ( $found ){
            update_post_meta( $a13_nava_title, 'a13_nava_page_slug', $a13_nava_page_slug );
        }
    }

    
}

add_action( 'save_post', 'apollo13_after_page_save' );


//for work
/*
 * Making cover for works in Works list
 */
if(!function_exists('a13_make_work_image')){
    function a13_make_work_image( $work_id, $thumb_size, $hidden = false ){
        if(empty($work_id)){
            $work_id = get_the_ID();
        }
        if ( has_post_thumbnail($work_id) ) {
            $src = wp_get_attachment_image_src( get_post_thumbnail_id ( $work_id ), $thumb_size );
            $src = $src[0];
        }
        else{
            $src = A13_TPL_GFX . '/holders/photo.jpg';
        }

        $style = '';
        if($hidden){
            $style = ' style="visibility: hidden;"';
        }

        return '<img src="'.esc_url($src).'" alt=""'.$style.' />';
    }
}


/*
 * For printing categories(taxonomies) of custom type post
 */
if(!function_exists('a13_cpt_work_posted_in')){
    function a13_cpt_work_posted_in( $separator = '<span>/</span>' ) {
        $term_list = wp_get_post_terms(get_the_ID(), A13_CPT_WORK_TAXONOMY, array("fields" => "all"));;
        $count_terms = count( $term_list );
        $html = '';
        $iter = 1;
        if( $count_terms ){
            foreach($term_list as $term) {
                $html .= '<a href="' . esc_url(get_term_link($term)) . '">' . $term->name . '</a>';
                if( $count_terms != $iter ){
                    $html .= $separator;
                }
                $iter++;
            }
        }

        return $html;
    }
}


/*
 * Collection of photos
 */
if(!function_exists('a13_make_media_collection')){
    function a13_make_media_collection(){
        global $a13_option;

        $ID             = get_the_ID();
        $theme          = a13_get_meta('_theme', $ID);
        $is_slider      = $theme === 'slider';
        $is_layer_slider= $theme === 'layer_slider';
        $is_scroller    = $theme === 'scroller';
        $is_full_photos = $theme === 'full_photos';
        $is_bricks      = $theme === 'bricks';
        $is_grid        = $theme === 'grid';
        $is_work        = defined( 'A13_WORK_PAGE' );
        $is_gallery     = defined( 'A13_GALLERY_PAGE' );
        $image_size     = 'full';
        $thumb_size     = 'full';
        $slides         = array();
        $media_count    = 0;
        $temp_attachment = '';

        //setup image and thumb sizes
        if( $is_gallery){
            if($is_slider){
                if($a13_option['galleries_slider_images'] === 'resized'){
                    $image_size = 'gallery-slider-image';
                }
                //no thumb size for this variant
            }
            elseif($is_bricks){
                $thumb_size = 'gallery-bricks';
            }
        }
        elseif( $is_work ){
            if($is_slider){
                if($a13_option['works_slider_images'] === 'resized'){
                    $image_size = 'work-slider-image';
                }
                $slider_height = get_post_meta($ID, '_slider_height', true);
                $slider_css = '';
                if( $slider_height > 0 ){
                    $slider_css = 'style="height: '.$slider_height.';"';
                }
                //no thumb size for this variant
            }
            elseif($is_scroller){
                if($a13_option['works_scroller_images'] === 'resized'){
                    $thumb_size = 'work-scroller-image';
                }
            }elseif($is_layer_slider){
                
                echo do_shortcode('[layerslider id="'.get_post_meta($ID, '_work_slider', true).'"]');
                return false;
            }
            
        }

        //default gallery settings
        $a_desc               = trim( get_post_meta($ID, '_description', true) );

        //get our items collection
        $images_videos_array = get_post_meta($ID, '_images_n_videos' , true);
        if(!empty($images_videos_array)){
            $images_videos_array = json_decode($images_videos_array, true);
            $media_count = count($images_videos_array);
        }
        
        if( $media_count ){
            //sorting order
            $order_asc = true;
            if($is_work || $is_gallery){
                $order = get_post_meta($ID, '_order', true);
                if($order === 'DESC'){
                    $order_asc = false;
                }
                elseif($order === 'random'){
                    shuffle($images_videos_array);
                }
            }

//            ASC
//            for( $i = 0; $i < $media_count; $i++ ){
//            DESC
//            for( $i = $media_count-1; $i >= 0; $i-- ){

            for( $i = ($order_asc? 0 : $media_count - 1); ($order_asc? ($i < $media_count) : ($i >= 0)); ($order_asc? $i++ : $i--) ){
                $current_element = $images_videos_array[$i];
                //image or video?
                $type = $current_element['item_type'];

                //image | video
                if( $type ){
                    $caption            = trim( $current_element['item_title'] );
                    $desc               = trim( $current_element['item_desc'] );
                    //local item settings VS gallery settings
                    $desc               = (empty($desc))? nl2br($a_desc) : nl2br($desc);

                    if($type === 'image'){
                        $media              = $current_element['item_image'];
                        //no image so we skip this item
                        if(!strlen($media)){
                            continue;
                        }

                        $attachment_id      = trim( $current_element['attachment_id'] );

                        //if not using full image version
                        if(!empty($attachment_id) && $image_size !== 'full'){
                            $temp = wp_get_attachment_image_src($attachment_id, $image_size);
                            if($temp){
                                $media = $temp[0];
                            }
                        }

                        $link               = trim( $current_element['item_link'] );
                        $bg_color           = trim( $current_element['bg_color'] );
                        $thumb              = $media;
                        $lbthumb            = $media;

                        //can get normal thumb ?
                        if(!empty($attachment_id)){
                            //main thumbnail
                            $temp_attachment = wp_get_attachment_image_src($attachment_id, $thumb_size);
                            if(!$temp_attachment){
                            }
                            else{
                                $thumb = $temp_attachment[0];
                            }

                            //lightbox thumb
                            $temp_attachment = wp_get_attachment_image_src($attachment_id, 'apollo-sidebar-size');
                            if(!$temp_attachment){
                            }
                            else{
                                $lbthumb = $temp_attachment[0];
                            }
                        }


                        //fill array with values
                        $slides[] = array(
                            'type'              => $type,
                            'image'             => $media,
                            'thumb'             => $thumb,
                            'lbthumb'           => $lbthumb,
                            'title'             => $caption,
                            'desc'              => $desc,
                            'bg_color'          => $bg_color,
                            'link'              => $link,
                        );
                    }

                    elseif($type === 'video'){
                        $media          = $current_element['item_video'];
                        if(!strlen($media)){
                            continue;
                        }

                        $v_arr          = a13_detect_movie($media);
                        $thumb          = a13_get_movie_thumb_src($v_arr, trim( $current_element['item_video_thumb'] ) );
                        $movie_link     = a13_get_movie_link($v_arr, array('width' => 500, 'height' => 500, 'poster' => $thumb));
                        $autoplay       = trim( $current_element['item_video_autoplay'] );

                        //fill array with values
                        $slides[] = array(
                            'type'              => $type,
                            'image'             => $movie_link,
                            'thumb'             => $thumb,
                            'lbthumb'           => $thumb,
                            'title'             => $caption,
                            'desc'              => $desc,
                            'autoplay'          => $autoplay,
                            'movie_type'        => $v_arr['type'],
                            'link'              => $media
                        );
                    }
                }
            }

            if( sizeof($slides) ){
                $html = '';
                $desc_html = '';
                $slide_i = 0;
                foreach($slides as $key => $slide){
                    //description
                    if(!empty($slide['desc'])){
                        $desc_html .= '<div id="g_d_'.$key.'">'.$slide['desc'].'</div>';
                    }
                    $slide_i ++;
                    if( $is_grid ){
                        $html .= '<div class="'.($slide_i % 3 == 1 ? 'full_grid_container' : ($slide_i % 3 == 2 ? 'half_grid_container1' : 'half_grid_container2' ) ).'">';
                        
                    }
                    //element
                    $html .= "\n".'<a class="' .
                        (($is_bricks || $is_slider)? 'g-link' : '' ) .
                        ($slide['type'] != 'image'? ' '.$slide['type'] : (empty($slide['link'])?'' : ' link')) .
                        '" href="'.esc_url(empty($slide['link'])? $slide['image'] : $slide['link']).'"'.
                        ' data-group="'.esc_attr('gallery'.$ID).'"'.
                        ((!empty($slide['autoplay']) && $slide['autoplay'] == '1')? ' data-autoplay="true"' : '') .
                        (!empty($slide['title'])? (' data-title="'.esc_attr(str_replace('"',"'",$slide['title'])).'"') : '') .
                        (!empty($slide['lbthumb'])? (' data-thumbnail="'.esc_attr($slide['lbthumb']).'"') : '') .
                        (!empty($slide['desc'])? (' data-description="'.esc_attr('#g_d_'.$key).'"') : '') .
                        (!empty($slide['movie_type'])? (' data-movie_type="'.esc_attr($slide['movie_type']).'"') : '') .
                        (!empty($slide['bg_color'])? (' data-bg_color="'.esc_attr($slide['bg_color']).'"') : '') .
                        ' data-image="'.esc_attr($slide['image']).'"' .
                        ' data-type="'.esc_attr($slide['type']).'"' .
                        '>'.
                        ($is_bricks? '<i>' : '' ) .
                        '<img src="'.esc_url($slide['thumb']).'" alt="" />' .
                        ($is_bricks? '</i>' : '' ) .
                        ((!$is_bricks && !$is_slider)? '<i></i>' : '' ) .
                        '</a>';
                    if( $is_grid ){
                        $html .= '</div>';
                        
                    }
                }

                $no_resize_param = '';
                if($is_bricks){
                    //class with info about resizing items
                    //needed cause of type of hover effect
                    $temp = $a13_option['galleries_brick_height'];
                    // if 0 then height if fluid
                    if(!($temp === '0px' || $temp === '0')){
                        $no_resize_param = ' data-no-resize="true"';
                    }
                }
                if($is_bricks || $is_slider){
                    $html = '<div id="a13-gallery"'.$no_resize_param.' data-autoplay="'.esc_attr(a13_get_meta( '_autoplay', false, 'works_autoplay' )).'" data-fit_variant="'.esc_attr(a13_get_meta( '_fit_variant' )).'" data-transition="'.esc_attr(a13_get_meta( '_transition', false, 'works_transition_type' )).'" class="'.esc_attr(a13_get_meta('_theme').' '.a13_get_meta('_hover_type')).'">'.$html.'</div>';
                    if(($is_work || $is_gallery) && $is_slider){
                        $html = '<div class="in-post-slider" '.$slider_css.'>'.$html.'</div>';
                    }
                    return $html . '<div id="g_descs">'.$desc_html.'</div>';
                }
                elseif($is_scroller){
                    return '<div id="a13-scroll-pan"><div id="a13-work-slides" class="clearfix '.esc_attr(a13_get_meta('_theme')).'">'.$html.'</div></div>'.
                        '<div id="g_descs">'.$desc_html.'</div>';
                }
                else{
                    return '<div id="a13-full-photos" class="'.( $is_grid ? 'grid' : 'list' ).'">'.$html.'</div>'.
                        '<div id="g_descs">'.$desc_html.'</div>';
                }

            }
            else{
                return false;
            }
        }
        else{  
            return false;               
        }
    }
}


// Initialize everything
$ceevee_plugin_base = new CeeVee_Plugin_Base();
