<?php

//for front-end translations
function _fe($str){
    _e($str, A13_TPL_SLUG);
}
function __fe($str){
    return __($str, A13_TPL_SLUG);
}

//for back-end translations
function _be($str){
    _e($str, A13_TPL_SLUG.'_admin');
}
function __be($str){
    return  __($str, A13_TPL_SLUG.'_admin');
}

/*
 * Check if current page is type of Posts list
 */
if(!function_exists('a13_is_post_list')){
    function a13_is_post_list(){
        return is_home() || (is_archive() && !defined('A13_WORKS_LIST_PAGE')) || is_search();
    }
}
/*
 * Pagination for blog pages
 */
if(!function_exists('a13_blog_nav')){
    function a13_blog_nav() {
        //if WP Painate plugin is installed and active
        if(function_exists('wp_paginate')) {
            wp_paginate();
        }
        //theme pagination
        else{
            global $paged, $wp_query;
            //safe copy for operations
            $c_paged = $paged;

            $max_page = $wp_query->max_num_pages;

            if ( $max_page > 1 ) : ?>
                <div id="posts-nav" class="navigation">
                    <?php
                    echo '<span class="nav-previous">';
                    previous_posts_link( __fe( 'Newer' ) );
                    echo '</span>';

                    
                    
                    echo '<span class="nav-next">';
                    next_posts_link( __fe( 'Older' ) );
                    echo '</span>';
                    ?>
                </div>
            <?php endif;
        }
    }
}
/*
 * Serves all post meta: author, date, comments, tags, categories
 */
if(!function_exists('a13_post_meta')){
    function a13_post_meta($paragraph_output = false) {
        global $a13_option;

        $single     = is_single();
        $post_list  = a13_is_post_list() || defined('A13_POST_LIST_SC');
        $work       = get_post_type() == 'work';
        $return     = '';

        //return date
        

        //return author
        if(
            ($single && $a13_option['post_meta_author'] == 1 )
            ||
            ($post_list && $a13_option['blog_meta_author'] == 1 )
        ){
            $return .= a13_posted_by_author();
        }

        //return categories
        if(
            ($single && $a13_option['post_meta_categories'] == 1 )
            ||
            ($post_list && $a13_option['blog_meta_categories'] == 1 )
        ){
            $return .= a13_post_categories().' ';
        }


        //return taxonomy for works
        if($work){
            $return .= a13_cpt_work_posted_in('');
        }

        //return comments number
        if(
            ($single && $a13_option['post_meta_comments_number'] == 1 )
            ||
            ($post_list && $a13_option['blog_meta_comments_number'] == 1 )
        ){
            $return .= a13_post_comments();
        }

        if(strlen($return)){
            if($paragraph_output){        
                echo '<p class="posted_in">'.$return.'</p>';
            }else{
                echo '<div class="post-meta">'.$return.'</div>';
            }
        }
    }
}


/*
 * Date of post
 */
if(!function_exists('a13_posted_on')){
    function a13_posted_on( $ribbon = true ) {
        //
        if( $ribbon ){
            return '<a href="#" class="a13_blog_ribbon blind_link"><div class="ribbon">
              <svg width="42" height="72">
                <path d="M 0,0 L 0,72 21,62 42,72 42,0 z"/>
              </svg>
              <div class="stripes"><i><span class="day">'.get_the_date('j').'</span><span class="month">'.get_the_date('M').'</span></i></div>
            </div></a>';
        }else{
            return '<span class="entry-date"><a href="'.esc_url(get_permalink()).'">'.get_the_date().'</a></span>';
        }
        
    }
}


/*
 * Author of post
 */
if(!function_exists('a13_posted_by_author')){
    function a13_posted_by_author() {
        return  '<span> '.__fe('by').' <a href="'.esc_url(get_author_posts_url( get_the_author_meta( 'ID' ) )).'" title="'.sprintf( esc_attr( __fe( 'View all posts by %s' ) ), get_the_author() ).'">'.get_the_author().'</a></span>';
    }
}


/*
 * comments & edit links
 */
if(!function_exists('a13_post_comments')){
    function a13_post_comments() {
        return '<span><span class="fa fa-comment"></span><a href="' . esc_url(get_comments_link()) . '" title="' .sprintf( esc_attr(__fe( '%d Comment(s)' )), get_comments_number()). '">'.sprintf(__fe( '%d Comment(s)' ), get_comments_number()).'</a></span>';
    }
}


/*
 * Post tags
 */
if(!function_exists('a13_post_tags')){
    function a13_post_tags() {
        $tags = '';
        $tag_list = get_the_tag_list( '',', ' );
        if ( $tag_list ) {
            $tags = '<span class="tag_list"><span class="fa fa-tags"></span>'.$tag_list.'</span>';
        }
        return $tags;
    }
}


/*
 * Categories that post was posted in
 */
if(!function_exists('a13_post_categories')){
    function a13_post_categories( ) {
        $cats = '';
        $cat_list = get_the_category_list(', ');
        if ( $cat_list ) {
            $cats = '<span>'.__fe('in').' '.$cat_list.'</span>'; 
        }

        return $cats;
    }
}


/*
 * Categories and Tags that post was posted in
 */
if(!function_exists('a13_posted_in')){
    function a13_posted_in() {
        global $a13_option;

        $return = '';
        $single = is_single();
        $post_list = a13_is_post_list();
        $work = get_post_type() == A13_CUSTOM_POST_TYPE_WORK;

        //return categories
        if(
            ($single && $a13_option['post_meta_categories'] == 1 )
            ||
            ($post_list && $a13_option['blog_meta_categories'] == 1 )
        ){
            $return .= a13_post_categories().' ';
        }

        //return tags
        if(
            ($single && $a13_option['post_meta_tags'] == 1 )
            ||
            ($post_list && $a13_option['blog_meta_tags'] == 1 )
        ){
            $return .= a13_post_tags();
        }

        //return taxonomy for works
        if($work){
            $return .= a13_cpt_work_posted_in(', ');
        }

        if(strlen(trim($return))) //trim if only space is present(page type in search result for example)
            $return = '<span class="posted-in">'.$return.'</span>';

        return $return;
    }
}
/*
 * Making images
 */
if(!function_exists('a13_make_post_image')){
    function a13_make_post_image( $post_id, $thumb_size){
        if(empty($post_id)){
            $post_id = get_the_ID();
        }
        if ( has_post_thumbnail($post_id) ) {
            return get_the_post_thumbnail($post_id, $thumb_size );
        }

        return false;
    }
}
/*
* Function that return featured image or video for post
*/
if(!function_exists('a13_top_image_video_slider')){
    function a13_top_image_video_slider($link_it = false, $width = 'auto', $height = 0){
        global $a13_option;

        $is_post = is_single();
        $is_page = is_page();
        $is_post_list = a13_is_post_list();

        //check if media should be displayed
        if(
            ($is_post && $a13_option['post_media'] == 0 )
            ||
            ($is_post_list && $a13_option['blog_post_media'] == 0 )
        )
            return;

        $post_id =  get_the_ID();
        $post_media_type = get_post_meta($post_id, '_post_media_type', true);
        
       
        $align = '';
        $size = '';

        if( empty( $post_media_type ) || $post_media_type == 'post_image' ){
            $thumb_size = 'apollo-post-thumb';

            if($is_post || $is_page){
                $align = ' '.get_post_meta($post_id, '_image_stretch', true);
                $size = get_post_meta($post_id, '_image_size', true);

                if($size == 'big') $thumb_size = 'apollo-post-thumb-big';
                elseif($size == 'original') $thumb_size = 'full';
                elseif($size == 'auto'){
                    if( defined('A13_FULL_WIDTH') || a13_get_meta( '_widget_area' ) == 'off'){
                        $thumb_size = 'apollo-post-thumb-big';
                    }
                }
            }
            elseif($is_post_list){
                
            }

            $img = a13_make_post_image($post_id, $thumb_size);

            if( !empty( $img ) ){
                if($link_it){
                    $img = '<a href="'.esc_url(get_permalink()).'">'.$img.'<em></em></a>';
                }
                ?>
                <div class="item-image <?php echo esc_attr('post-media'.$align); ?>">
                    <?php echo $img; ?>
                </div>
                <?php
            }else{
                ?>
                <div class="empty-item-image "></div>
                <?php
            }
        }

        elseif( $post_media_type == 'post_video' ){
            if($width === 'auto'){
                $width = 600;
            }

            if($is_post || $is_page){
                $align = ' '.get_post_meta($post_id, '_video_align', true);
                $size = get_post_meta($post_id, '_video_size', true);
                if($size == 'big') $width = 960;
            }

            $src = get_post_meta($post_id, '_post_video', true);
            if( !empty( $src ) ){
                ?>
                <div class="item-video post-media <?php echo esc_attr('width-'.$size.$align); ?>">
                    <?php
                    if( $height == 0){
                        $height = ceil((9/16) * $width);
                    }

                    $v_code = wp_oembed_get($src, array(
                        'width' => $width,
                        'height' => $height
                        )
                    );

                    //if no code, try theme function
                    if($v_code === false){
                        echo a13_get_movie($src, $width, $height);
                    }
                    else{
                        echo $v_code;
                    }
                    ?>
                </div>
                <?php
            }
        }
    }
}

/* Generates user css based on settings in admin panel */
function apollo13_make_css_rule($property, $value, $format = false){
    if ( $value !== '' &&  $value !== 'default' ){
        //make fallback for rgba colors by providing rgb color
        if(strpos($property,'color') !== false && strpos($value,'rgba') !== false){
            //break rgba to numbers
            $chunks = a13_break_rgba($value);
            $fallback_color = "rgb($chunks[1], $chunks[2], $chunks[3])";

            return
                $property . ': ' . $fallback_color . "; " .
                $property . ': ' . $value . ';';
        }
        

        //format for some properties
        if( $format !== false ){
            return $property . ': ' . sprintf($format, $value) . ';';
        }

        return $property . ': ' . $value . ";";
    }
    else{
        return '';
    }
}

/* only for background color
 * it gives RGBA possibility to IE 8
*/
function apollo13_ie_color($property, $value){
    if(strpos($value,'rgba') !== false){
        //break rgba to numbers
        $chunks = a13_break_rgba($value);

        $ie_color = a13_rgba2hex($chunks[1], $chunks[2], $chunks[3], $chunks[4]);

        $css =
            'background-color: transparent; ' .
                'filter:progid:DXImageTransform.Microsoft.gradient(startColorStr='.$ie_color.',endColorStr='.$ie_color.'); ' .
                'zoom: 1;';

        return $css;
    }
    else{
        return apollo13_make_css_rule($property, $value);
    }
}

function a13_rgba2hex( $r, $g, $b, $a ){
    return sprintf( '#%02s%02s%02s', dechex( $r ), dechex( $g ), dechex( $b ) );
}

function a13_break_rgba( $rgba ){
    $chunks = array();
    preg_match("/\(\s*(\d+),\s*(\d+),\s*(\d+),\s*(\d+\.?\d*)\s*\)/", $rgba, $chunks);
    return $chunks;
}
/*
 * Prints favicon
 */
if(!function_exists('a13_favicon')){
    function a13_favicon() {
        global $a13_option;
        if( isset($a13_option['layout_favicon']) ){
            $fav_icon = $a13_option['layout_favicon']['url'];
            if(!empty($fav_icon))
                echo '<link rel="shortcut icon" href="'.esc_url($fav_icon).'" />';
        }
        
    }
}
/*
 * Gets logo image and check for HIGH DPI cookie
 */
if(!function_exists('a13_preloader_image')){
    function a13_preloader_image() {
        global $a13_option;
        $style = $temp ="";
        $src = ( isset( $a13_option['layout_preloader_icon'] ) ? $a13_option['layout_preloader_icon']['url'] : '' );
        $screen_type = isset($_COOKIE["a13_screen_size"])? $_COOKIE["a13_screen_size"] : 'normal';

        if($screen_type === 'high' && isset( $a13_option['layout_preloader_retina_icon'] ) ){
            $temp = $a13_option['layout_preloader_retina_icon'];
            if(strlen($temp['url'])){
                $src = $temp['url'];
                $style = 'width:'.ceil(intval($temp['width'])/2).'px;height:'.ceil(intval($temp['height'])/2).'px;';
            }
        }
        if( strlen( $src ) ){
            echo '<img src="'.esc_url($src).'" style="'.$style.'" alt="'. esc_attr( get_bloginfo( 'name', 'display' ) ).'" />';
        }
    }
}

/*
 * Gets logo image and check for HIGH DPI cookie
 */
if(!function_exists('a13_header_logo_image')){
    function a13_header_logo_image() {
        global $a13_option;
        $style = $temp ="";
                        
        $src = ( isset( $a13_option['layout_logo_image1'] ) ? $a13_option['layout_logo_image1']['url'] : '' );
        $forced_width = ( isset( $a13_option['layout_logo_image1']['width'] ) ? $a13_option['layout_logo_image1']['width'] : '' );
        
        $screen_type = isset($_COOKIE["a13_screen_size"])? $_COOKIE["a13_screen_size"] : 'normal';

        if($screen_type === 'high' && isset( $a13_option['layout_logo_retina_image1'] ) ){

            $temp = $a13_option['layout_logo_retina_image1'];
            if(strlen($temp['url'])){
                $src = $temp['url'];
                
                $width = ($forced_width);
                $height = ceil(intval($temp['height'])/2);
                if($width>$height){
                    $height = 'auto';
                    $width .= 'px';
                }
                else{
                    $width = 'auto';
                    $height .= 'px';
                }
                $style = 'width:'.$width.';height:'.$height.';';
                
            }
        }
        if( strlen( $src ) ){
            echo '<img class="logo_img1" src="'.esc_url($src).'" style="'.$style.'" alt="'. esc_attr( get_bloginfo( 'name', 'display' ) ).'" />';
        }
        
        
    }
}

/*
 * Prints side menu for static pages that has parents or children
 */
if(!function_exists('a13_page_menu')){
    function a13_page_menu($only_check = false) {
        global $post;

        $there_is_menu = false;

        $has_children_args = array(
            'post_parent' => $post->ID,
            'post_status' => 'publish',
            'post_type' => 'any',
        );

        $list_pages_params = array(
            'child_of'      => $post->post_parent,
            'sort_column'   => 'menu_order',
            'depth'         => 0,
            'title_li'      => '',
            'walker'        => new A13_list_pages_walker
        );

        if(a13_is_subpage()){
            if($only_check){ return true; }
            $there_is_menu = true;
        }
        elseif(get_children( $has_children_args )){
            if($only_check){ return true; }
            $list_pages_params['child_of'] = $post->ID;
            $there_is_menu = true;
        }

        //display menu
        if($there_is_menu){
            echo '<div class="widget a13_page_menu widget_nav_menu">
                    <ul>';

                wp_list_pages($list_pages_params);

            echo '</ul>
                </div>';
        }

        return false;
    }
}
/*
* Checks if current page has active sidebar
 * returns false if there is no active sidebar,
 * if there is active sidebar it returns its name
*/
if(!function_exists('a13_has_active_sidebar')){
    function a13_has_active_sidebar() {
        global $a13_option;
        $test = '';
        if( is_home() || is_archive() || is_search() || defined('A13_NO_STYLED_PAGE') ){
            $test = 'blog-widget-area';
        }
        elseif(defined('A13_WORK_PAGE')){}
        elseif( is_single() && a13_get_meta('_widget_area',false,'post_sidebar') != 'off' ){
            $test = 'post-widget-area';
        }
        

        if( is_active_sidebar($test)){
            return $test;
        }
        else{
            return false;
        }
    }
}

/*
* Get classes for body element
*/
if(!function_exists('a13_body_classes')){
    function a13_body_classes( $classes ) {
        global $a13_option;

        //background fit
        $classes[] = $a13_option['layout_background_image_fit'];

        //protected album
        if(defined('A13_PAGE_PROTECTED')){
            $classes[] = 'password-protected';
        }

        if(defined('A13_WORK_PAGE')){
        }

        if(defined('A13_WORKS_LIST_PAGE')){
            $classes[] = 'works-list-page';
        }

        if(defined('A13_GALLERIES_LIST_PAGE')){
            $classes[] = 'galleries-list-page';
        }

        //no results page
        if(defined('A13_NO_STYLED_PAGE'))
            $classes[] = 'no-results';

        return $classes;
    }
}
/*
 * Returns array with type of current page
 */
if(!function_exists('a13_what_page_type_is_it')){
    function a13_what_page_type_is_it() {
        static $types;

        if ( empty( $types ) ) {
            $types = array(
                'page'          => is_page(),
                'work'          => defined('A13_WORK_PAGE'),
                'gallery'       => defined('A13_GALLERY_PAGE'),
                'home'          => is_home(),
                'front_page'    => is_front_page(),
                'archive'       => is_archive(),
                'search'        => is_search(),
                'single'        => is_single(),
                'attachment'    => is_attachment(),
                'works_list'    => defined('A13_WORKS_LIST_PAGE'),
                'galleries_list'=> defined('A13_GALLERIES_LIST_PAGE')
            );

            $types['single_not_post']   = $types['page'] || $types['work'] || $types['gallery'];
            $types['blog_type']         = $types['home'] || $types['archive'] || $types['search'];
            $types['cpt']               = $types['work'] || $types['gallery'];
            $types['cpt_list']          = $types['works_list'] || $types['galleries_list'];
        }

        return $types;
    }
}
/*
* Return HTML for social icons
*/
if(!function_exists('a13_social_icons')){
    function a13_social_icons(){
        global $a13_option;

        $socials = array(
    		'500px'       => $a13_option['social-500px'],
    		'aim'         => $a13_option['social-aim'],
    		'behance'     => $a13_option['social-behance'],
    		'blogger'     => $a13_option['social-blogger'],
    		'delicious'   => $a13_option['social-delicious'],
    		'deviantart'  => $a13_option['social-deviantart'],
    		'digg'        => $a13_option['social-digg'],
    		'dribbble'    => $a13_option['social-dribbble'],
    		'facebook'    => $a13_option['social-facebook'],
    		'flickr'      => $a13_option['social-flickr'],
    		'forrst'      => $a13_option['social-forrst'],
    		'foursquare'  => $a13_option['social-foursquare'],
    		'github'      => $a13_option['social-github'],
    		'googleplus'  => $a13_option['social-googleplus'],
    		'instagram'   => $a13_option['social-instagram'],	
    		'lastfm'      => $a13_option['social-lastfm'],
            'linkedin'    => $a13_option['social-linkedin'],
    		'paypal'      => $a13_option['social-paypal'],		
    		'pinterest'   => $a13_option['social-pinterest'],		
    		'quora'       => $a13_option['social-quora'],		
    		'rss'         => $a13_option['social-rss'],	
            'reddit'      => $a13_option['social-reddit'],	
    		'sharethis'   => $a13_option['social-sharethis'],		
    		'skype'       => $a13_option['social-skype'],	
            'spotify'     => $a13_option['social-spotify'],		
    		'stumbleupon' => $a13_option['social-stumbleupon'],		
    		'tumblr'      => $a13_option['social-tumblr'],		
    		'twitter'     => $a13_option['social-twitter'],		
    		'vimeo'       => $a13_option['social-vimeo'],
    		'wordpress'   => $a13_option['social-wordpress'],
    		'yahoo'       => $a13_option['social-yahoo'],
            'yelp'        => $a13_option['social-yelp'],
            'youtube'     => $a13_option['social-youtube']
            );		
    	//print_r($socials);
        //uasort($socials, "a13_cmp_socials_positions");
        $soc_html = '';
        $has_active = false;
        $protocols = wp_allowed_protocols();
        $protocols[] = 'skype';
    
        foreach( $socials as $id => $value ){
            if( ! empty($value) ){
                $soc_html .= '';
                $soc_html .= '<a data-path-hover="M 0,0 L 0,62 21,70 42,62 42,0 z" target="_blank" href="' . esc_url($value, $protocols) . '" title="' . esc_attr(__fe( 'Follow us on ' ) . ucfirst( $id )) . '" class="a13_soc-'.$id.'"><div class="ribbon">
  <svg width="42" height="72">
    <path d="M 0,0 L 0,60 21,50 42,60 42,0 z"/>
  </svg>
  <div class="stripes"><i><span></span></i></div>
</div></a>';
                $soc_html .= '';
                $has_active = true;
            }
        }
    
        if($has_active){
            $soc_html = '<div class="socials social_ribbon">'.$soc_html.'</div>';
        }
    
        return $soc_html;
    }
}

function a13_return_soc_icons($socials){

    $soc_html = '';
    $has_active = false;
    $protocols = wp_allowed_protocols();
    $protocols[] = 'skype';

    foreach( $socials as $id => $value ){
        if( ! empty($value) ){
            $soc_html .= '<a target="_blank" href="' . esc_url($value, $protocols) . '" title="' . esc_attr(__fe( 'Follow us on ' ) . ucfirst( $id )) . '" class="'.esc_attr('a13_soc-'.$id).'"></a>';
            $has_active = true;
        }
    }

    if($has_active){
        $soc_html = '<div class="socials">'.$soc_html.'</div>';
    }

    return $soc_html;
}

/*
* Get classes for mid element
*/
function a13_set_full_width(&$mid_classes){
    global $content_width;
    define('A13_FULL_WIDTH', true); /* so we don't have to check again in sidebar.php */
    $mid_classes .= ' no-sidebars';
    //content width
    $content_width = 1080;
}
function a13_set_sidebar_class(&$mid_classes, $sidebar){
    if(($sidebar == 'off')){
        a13_set_full_width($mid_classes);
    }
    else{
        $mid_classes .= ' '.$sidebar;
    }
}
if(!function_exists('a13_get_mid_classes')){
    function a13_get_mid_classes() {
        global $a13_option, $content_width;

        //mid classes for type of layout align and widget area display(on/off)
        $mid_classes = '';
        //404 error page, no-result page, etc.
        $is_empty_page = defined('A13_NO_STYLED_PAGE');

        $page_type = a13_what_page_type_is_it();
//        var_dump($page_type);
        $page = $page_type['page'];

        //check if there is active sidebar for current page
        $force_full_width = false;
        if( $page_type['cpt_list'] || //it is page, so it can gain page sidebar
            $page_type['cpt']       || //it doesn't have sidebar
            a13_has_active_sidebar() === false
        ){
            $force_full_width = true;
        }

    

        /*
         * sidebar classes
         * */
        if($page){
            a13_set_full_width($mid_classes);
            $mid_classes .= ' full-width-elements';
        }
        elseif($force_full_width){
        
            a13_set_full_width($mid_classes);
        }
        //blog | attachment | or empty page
        elseif($page_type['archive'] || $page_type['search'] || $page_type['home'] || $page_type['attachment'] || $is_empty_page){
            a13_set_sidebar_class($mid_classes, $a13_option['blog_sidebar']);
        }
        
        //single post
        elseif($page_type['single']){
            a13_set_sidebar_class($mid_classes, a13_get_meta('_widget_area',false,'post_sidebar'));
        }
        
        return $mid_classes;
    }
}

/*
* Displays mobile menu
*/
if(!function_exists('a13_mobile_menu')){
    function a13_mobile_menu(){
        /* Our navigation menu.  If one isn't filled out, wp_nav_menu falls back to wp_page_menu.
         * The menu assiged to the primary position is the one used.
         * If none is assigned, the menu with the lowest ID is used.
         */
        if ( has_nav_menu( 'header-menu' ) ):
            wp_nav_menu( array(
                    'container'       => false,
                    'link_before'     => '<span>',
                    'link_after'      => '</span>',
                    'menu_class'      => 'mobile-menu',
                    'menu_id'         => 'mobile_menu',
                    'theme_location'  => 'header-menu',
                    'walker'          => new A13_mmenu_walker)
            );
        else:
            echo '<ul class="mobile-menu">';
            wp_list_pages(
                array(
                    'link_before'     => '<span>',
                    'link_after'      => '</span>',
                    'title_li' 		  => ''
                )
            );
            echo '</ul>';
        endif;
    }
}

/*
* Displays header menu
*/
if(!function_exists('a13_header_menu')){
    function a13_header_menu(){
        /* Our navigation menu.  If one isn't filled out, wp_nav_menu falls back to wp_page_menu.
         * The menu assiged to the primary position is the one used.
         * If none is assigned, the menu with the lowest ID is used.
         */
        if ( has_nav_menu( 'header-menu' ) ):
            wp_nav_menu( array(
                    'container'       => false,
                    'link_before'     => '<span>',
                    'link_after'      => '</span>',
                    'menu_class'      => 'top-menu',
                    'theme_location'  => 'header-menu',
                    'walker'          => new A13_menu_walker)
            );
        else:
            echo '<ul class="top-menu">';
            wp_list_pages(
                array(
                    'link_before'     => '<span>',
                    'link_after'      => '</span>',
                    'title_li' 		  => ''
                )
            );
            echo '</ul>';
        endif;
    }
}

class A13_menu_walker extends Walker_Nav_Menu {

    /**
     * @see Walker::start_el()
     * @since 3.0.0
     *
     * @param string $output Passed by reference. Used to append additional content.
     * @param object $item Menu item data object.
     * @param int $depth Depth of menu item. Used for padding.
     * @param object $args
     * @param int $id Menu item ID.
     */
    function start_el( &$output, $item, $depth = 0, $args = array(), $id = 0 ) {
        global $wp_query;
        $indent = ( $depth ) ? str_repeat( "\t", $depth ) : '';

        $fa_class = $item_slug = $class_names = $value = '';

        $classes = empty( $item->classes ) ? array() : (array) $item->classes;
        
        if( strpos($item->classes[0],'fa-')!== false ){
            $fa_class = $item->classes[0] . ' a13_menu_icon';
        }
        //special case - items with anchor href
        $hash_index = strpos($item->url,'#');
        $home_page_id = get_option('page_on_front');
        $frontpage = get_post( $home_page_id );
        if ( get_post() )
        {
            //instead of using get_the_ID() - it's broken...
            $id_of_displayed_post = get_the_ID();
        }else{
            $id_of_displayed_post = null;
        }
        
        if ( !is_404() && ( $item->object == 'nava' || ( $id_of_displayed_post == $item->object_id ) ) ){
            $item_anchor = '';
            if ( $item->object == 'nava' ){
                $nava_post_meta = get_post_meta( $item->object_id );
                $item_slug = $nava_post_meta['a13_nava_page_slug'][0];
                $item_anchor = 'a13_'.$item->title;
            }
            
            $current_slug = $wp_query->post->post_name;
            //for pages in menu - generates scroll to top
            if ( $id_of_displayed_post == $item->object_id && $item_slug == '' ){
                $current_slug = $item_slug = '';
                $item_anchor = 'page_very_top';
            }
            if ( $current_slug == $item_slug ){
                //$item->url = str_replace( $current_slug.'#'.$anchor, '#'.$anchor, $item->url );
                $item->url = '#'.$item_anchor;                
                $classes[] = 'scroll-to'; 
                
            }elseif( $item_slug !='' ){
                if ( $frontpage->post_name == $item_slug){
                    $item_slug = get_site_url();
                }
                $item->url = $item_slug . '#' . $item_anchor;
            }
            
        }
        
        if( is_front_page() and $home_page_id == $item->object_id ){
            $classes[] = 'active';
        }
        $classes[] = 'menu-item-' . $item->ID;

        //checks if this element is parent element
        $is_parent = (bool)array_search('menu-parent-item', $classes);
        $is_child = $item->menu_item_parent > 0;
        
        if( $is_child ){
            $fa_class .= ' a13_menu_icon';
        }                       

        $class_names = join( ' ', apply_filters( 'nav_menu_css_class', array_filter( $classes ), $item, $args ) );
        $class_names = $class_names ? ' class="' . esc_attr( $class_names ) . '"' : '';

        $id = apply_filters( 'nav_menu_item_id', 'menu-item-'. $item->ID, $item, $args );
        $id = $id ? ' id="' . esc_attr( $id ) . '"' : '';
        
        

        $output .= $indent . '<li' . $id . $value . $class_names .'>';
        
        

        $attributes  = ! empty( $item->attr_title ) ? ' title="'  . esc_attr( $item->attr_title ) .'"' : '';
        $attributes .= ! empty( $item->target )     ? ' target="' . esc_attr( $item->target     ) .'"' : '';
        $attributes .= ! empty( $item->xfn )        ? ' rel="'    . esc_attr( $item->xfn        ) .'"' : '';
        $attributes .= ! empty( $item->url )        ? ' href="'   . esc_attr( $item->url        ) .'"' : '';

        $item_output = $args->before;
        $item_output .= '<a class="'.$fa_class.'" '. $attributes .' title="'.$item->title.'" >';
        $item_output .= $args->link_before . apply_filters( 'the_title', $item->title, $item->ID ) . $args->link_after;
        $item_output .= $is_parent? '<i class="arr fa '.($depth === 0? 'fa-angle-down' : 'fa-angle-right').'"></i>' : '';
        $item_output .= '</a>';
        $item_output .= $args->after;

        $output .= apply_filters( 'walker_nav_menu_start_el', $item_output, $item, $depth, $args );
    }
}

class A13_mmenu_walker extends Walker_Nav_Menu {

    /**
     * @see Walker::start_el()
     * @since 3.0.0
     *
     * @param string $output Passed by reference. Used to append additional content.
     * @param object $item Menu item data object.
     * @param int $depth Depth of menu item. Used for padding.
     * @param object $args
     * @param int $id Menu item ID.
     */
    function start_el( &$output, $item, $depth = 0, $args = array(), $id = 0 ) {
        global $wp_query;
        $indent = ( $depth ) ? str_repeat( "\t", $depth ) : '';

        $item_slug = $class_names = $value = '';
        if( strpos($item->classes[0],'fa-')!== false ){
            unset($item->classes[0]);
        }
        $classes = empty( $item->classes ) ? array() : (array) $item->classes;
        
        
        //special case - items with anchor href
        $hash_index = strpos($item->url,'#');
        $home_page_id = get_option('page_on_front');
        $frontpage = get_post( $home_page_id );
        if ( get_post() )
        {
            //instead of using get_the_ID() - it's broken...
            $id_of_displayed_post = get_the_ID();
        }else{
            $id_of_displayed_post = null;
        }
        
        if ( !is_404() && ( $item->object == 'nava' || ( $id_of_displayed_post == $item->object_id ) ) ){
            $item_anchor = '';
            if ( $item->object == 'nava' ){
                $nava_post_meta = get_post_meta( $item->object_id );
                $item_slug = $nava_post_meta['a13_nava_page_slug'][0];
                $item_anchor = 'a13_'.$item->title;
            }
            
            $current_slug = $wp_query->post->post_name;
            //for pages in menu - generates scroll to top
            if ( $id_of_displayed_post == $item->object_id && $item_slug == '' ){
                $current_slug = $item_slug = '';
                $item_anchor = 'page_very_top';
            }
            if ( $current_slug == $item_slug ){
                //$item->url = str_replace( $current_slug.'#'.$anchor, '#'.$anchor, $item->url );
                $item->url = '#'.$item_anchor;                
                $classes[] = 'scroll-to'; 
                
            }elseif( $item_slug !='' ){
                if ( $frontpage->post_name == $item_slug){
                    $item_slug = get_site_url();
                }
                $item->url = $item_slug . '#' . $item_anchor;
            }
            
        }
        
        if( is_front_page() and $home_page_id == $item->object_id ){
            $classes[] = 'active';
        }
        $classes[] = 'menu-item-' . $item->ID;

        //checks if this element is parent element
        $is_parent = (bool)array_search('menu-parent-item', $classes);

        $class_names = join( ' ', apply_filters( 'nav_menu_css_class', array_filter( $classes ), $item, $args ) );
        $class_names = $class_names ? ' class="' . esc_attr( $class_names ) . '"' : '';

        $id = apply_filters( 'nav_menu_item_id', 'menu-item-'. $item->ID, $item, $args );
        $id = $id ? ' id="' . esc_attr( $id ) . '"' : '';
        
        

        $output .= $indent . '<li' . $id . $value . $class_names .'>';
        
        

        $attributes  = ! empty( $item->attr_title ) ? ' title="'  . esc_attr( $item->attr_title ) .'"' : '';
        $attributes .= ! empty( $item->target )     ? ' target="' . esc_attr( $item->target     ) .'"' : '';
        $attributes .= ! empty( $item->xfn )        ? ' rel="'    . esc_attr( $item->xfn        ) .'"' : '';
        $attributes .= ! empty( $item->url )        ? ' href="'   . esc_attr( $item->url        ) .'"' : '';

        $item_output = $args->before;
        $item_output .= '<a'. $attributes .'>';
        $item_output .= $args->link_before . apply_filters( 'the_title', $item->title, $item->ID ) . $args->link_after;
        $item_output .= $is_parent? '<i class="arr fa '.($depth === 0? 'fa-angle-down' : 'fa-angle-right').'"></i>' : '';
        $item_output .= '</a>';
        $item_output .= $args->after;

        $output .= apply_filters( 'walker_nav_menu_start_el', $item_output, $item, $depth, $args );
    }
}
/*
* Retrieves meta with checking for parent settings, and global settings
*/
function a13_get_meta($field, $post_id = false, $global_option = ''){
    $meta = '';
    $id = $post_id;
    
    global $a13_option;
    if(!is_404()){
        if(!$id){
            $id = get_the_ID();
        }
        $meta_tmp = trim(get_post_meta($id, $field, true));

        if ( ($meta_tmp == 'G' || $meta_tmp == '-1') && isset( $a13_option[$global_option] ) && $a13_option[$global_option] != '' ){
            $meta = $a13_option[$global_option];
            
        }else{
            $meta = $meta_tmp;
        }
    }

    if($id){

        return $meta;
    }
        return false;
}


/*
 * Prints search form with custom id for each displayed form one one page
 */
if(!function_exists('a13_search_form')){
    function a13_search_form() {
        global $a13_option;
        static $search_id = 1;
        $field_search = '';
        $helper_search = get_search_query() == '' ? true : false;
        $placeholder = ( '' != $a13_option['layout_header_search_text'] ? $a13_option['layout_header_search_text'] : __fe('Enter Search Words and Press Enter ...' ));
        $field_search = '<input' .
            ' placeholder="' . esc_attr($placeholder) . '" ' .
            'type="search" name="s" id="s' . $search_id . '" value="' .
            esc_attr( $helper_search ? '' : get_search_query() ) .
            '" />';

        $form = '
                <form class="search-form" role="search" method="get" action="' . home_url( '/' ) . '" >
                    <fieldset class="semantic">
                        <span class="fa fa-search"></span>
                        ' . $field_search . '
                        <input type="submit" id="searchsubmit' . $search_id . '" title="'. esc_attr( __fe('Search' ) ) .'" value=" " />
                    </fieldset>
                </form>';

        //next call will have different ID
        $search_id++;
        return $form;
    }
}
/*
 * Prints search form with custom id for each displayed form one one page
 */
if(!function_exists('a13_default_search_form')){
    function a13_default_search_form() {
        global $a13_option;
        static $search_id = 1;
        $field_search = '';
        $helper_search = get_search_query() == '' ? true : false;
        $placeholder = __fe('Type and press enter ...' );
        $field_search = '<input ' .
            ' placeholder="' . esc_attr($placeholder) . '" ' .
            'type="search" name="s" id="s' . $search_id . '" value="' .
            esc_attr( $helper_search ? '' : get_search_query() ) .
            '" />';

        $form = '
                <form class="widget-search-form" role="search" method="get" action="' . home_url( '/' ) . '" >
                    
                        
                        ' . $field_search . '
                        <span class="fa fa-search"></span>
                   
                </form>';

        //next call will have different ID
        $search_id++;
        return $form;
    }
}
/*
 * Header search form
 */
if(!function_exists('a13_header_search')){
    function a13_header_search() {
        echo
            '<div class="search-container">'.
                '<div class="search">'.
                    '<span class="fa fa-search open action"></span>'.a13_search_form();
                    //get_search_form();
        echo
                    '<span class="fa fa-times close action"></span>'.
                '</div>'.
            '</div>';
    }
}


/*
 * Detection of type of movie
 * returns array(type, video_id)
 */
if(!function_exists('a13_detect_movie')){
    function a13_detect_movie($src){
        //used to check if it is audio file
        $parts = pathinfo($src);
        $ext = isset($parts['extension'])? strtolower($parts['extension']) : false;

        //http://www.youtube.com/watch?v=e8Z0YTWDFXI
        if (preg_match("/(youtube\.com\/watch\?)?v=([a-zA-Z0-9\-_]+)&?/s", $src, $matches)){
            $type = 'youtube';
            $video_id = $matches[2];
        }
        //http://youtu.be/e8Z0YTWDFXI
        elseif (preg_match("/(https?:\/\/youtu\.be\/)([a-zA-Z0-9\-_]+)&?/s", $src, $matches)){
            $type = 'youtube';
            $video_id = $matches[2];
        }
        // regexp $src http://vimeo.com/16998178
        elseif (preg_match("/(vimeo\.com\/)([0-9]+)/s", $src, $matches)){
            $type = 'vimeo';
            $video_id = $matches[2];
        }
        elseif(strlen($ext) && in_array($ext, array('mp3', 'ogg', 'm4a'))){
            $type = 'audio';
            $video_id = $src;
        }
        else{
            $type = 'html5';
            $video_id = $src;
        }

        return array(
            'type' => $type,
            'video_id' => $video_id
        );
    }
}


/*
 * Returns movie thumb(for youtube, vimeo)
 */
if(!function_exists('a13_get_movie_thumb_src')){
    function a13_get_movie_thumb_src( $video_res, $thumb = '' ){
        if(!empty($thumb)){
            return $thumb;
        }

        $type = $video_res['type'];
        $v_id = $video_res['video_id'];

        if ( $type == 'youtube' ){
            return 'http://img.youtube.com/vi/'.$v_id.'/hqdefault.jpg';
        }
        elseif ( $type == 'vimeo' ){
            return A13_TPL_GFX . '/holders/vimeo.jpg';
        }
        elseif ( $type == 'html5' ){
            return A13_TPL_GFX . '/holders/video.jpg';
        }

        return false;
    }
}


/*
 * Returns movie link to insert it in iframe
 */
if(!function_exists('a13_get_movie_link')){
    function a13_get_movie_link( $video_res, $params = array()){
        $type = $video_res['type'];
        $v_id = $video_res['video_id'];

        if ( $type == 'youtube' ){
            return 'http://www.youtube.com/embed/'.$v_id.'?enablejsapi=1&amp;controls=1&amp;fs=1&amp;hd=1&amp;loop=0&amp;rel=0&amp;showinfo=1&amp;showsearch=0&amp;wmode=transparent';
        }
        elseif ( $type == 'vimeo' ){
            return 'http://player.vimeo.com/video/'.$v_id.'?api=1&amp;title=1&amp;loop=0';
        }
        else{
            return A13_TPL_ADV . '/inc/videojs/player.php?src=' . $v_id . '&amp;w=' . $params['width'] . '&amp;h=' . $params['height'] . '&amp;poster=' . $params['poster'];
        }
    }
}
/*
 * Returns movie iframe or link to movie
 */
if(!function_exists('a13_get_movie')){
    function a13_get_movie( $src, $width = 295, $height = 0 ){
        if( $height == 0){
            $height = ceil((9/16) * $width);
        }

        $video_res  = a13_detect_movie($src);
        $type       = $video_res['type'];
        if($type == 'audio'){
            return a13_get_audio_player($video_res['video_id']);
        }
        else{
            $link       = a13_get_movie_link($video_res, array( 'width' => $width, 'height' => $height, 'poster' => "" ));

            return '<iframe data-vid-id="'.$video_res['video_id'].'" id="crazy'.$type . mt_rand() . '" style="height: ' . $height . 'px; width: ' . $width . 'px; border: none;" src="' . esc_url($link) . '" webkitAllowFullScreen mozallowfullscreen allowFullScreen></iframe>';
        }
    }
}

add_filter( 'get_search_form','a13_default_search_form' );
add_filter( 'body_class', 'a13_body_classes' );

/*
* It replaces WP default action while closing children comments block
* Useful to save your nerves
*/
if(!function_exists('a13_comment_end')){
    function a13_comment_end( $comment, $args, $depth ) {
        echo '</div>';
        return;
    }
}


/*
* Changes default comment template
* Closing </div> for this block is produced by comment_end()
* It is strange, I know :-)
*/
if(!function_exists('a13_comment')){
    function a13_comment( $comment, $args, $depth ) {
        $GLOBALS['comment'] = $comment;

        switch ( $comment->comment_type ) :
            case '' :
                ?>
                    <div <?php comment_class( 'comment-block' ); ?> id="comment-<?php comment_ID(); ?>">

                        <a class="avatar" href="<?php esc_url(get_comment_author_url()); ?>" title=""><?php echo get_avatar( $comment, 40 ) ; ?></a>
                        <div class="comment-inside">
                            <div class="comment-info">
                                <span class="author"><?php comment_author_link(); ?></span>
                                <?php
                                    printf( 'on <a class="time" href="%1$s"><time datetime="%2$s">%3$s</time></a>',
                                        esc_url( get_comment_link( $comment->comment_ID ) ),
                                        esc_attr(get_comment_time( 'c' )),
                                        /* translators: 1: date, 2: time */
                                        sprintf( __fe( '%1$s at %2$s' ), get_comment_date('F j, Y'), get_comment_time('g:i a') )
                                    );
                                    comment_reply_link( array_merge( $args, array( 'before' => ' <span class="sepa">/</span> ', 'depth' => $depth, 'max_depth' => $args['max_depth'] ) ) );
                                ?>
                            </div>
                            <div class="comment-text">
                                <?php if ( $comment->comment_approved == '0' ) : ?>
                                <p><em class="comment-awaiting-moderation"><?php _fe( 'Your comment is awaiting moderation.' ); ?></em></p>
                                <?php endif; ?>
                                <?php comment_text(); ?>
                            </div>
                            <div class="clear"></div>
                        </div>
                    <?php
                break;
            case 'pingback'  :
            case 'trackback' :
                ?>
            <div <?php comment_class( 'comment-block' ); ?> id="comment-<?php comment_ID(); ?>">
                <div class="comment-inside clearfix">
                    <p><?php _fe( 'Pingback:' ); ?> <?php comment_author_link(); ?><?php edit_comment_link( '(' . __fe( 'Edit' ) . ')', ' ' ); ?></p>
                </div>
                    <?php
                break;
        endswitch;
    }
}

if(!function_exists('a13_similar_posts')){
    function a13_similar_posts(){
        global $apollo13, $post;

        $is_work = defined('A13_WORK_PAGE');
        $widget_title = __be('Similar Posts');

        if($is_work){
            $__search = wp_get_post_terms(get_the_ID(), A13_CPT_WORK_TAXONOMY, array("fields" => "slugs"));
            $widget_title = __be('Similar Works');
        }
        else{
            $__search = wp_get_post_tags($post->ID);
            $search_string = 'tags__in';
            //if no tags try categories
            if( !count($__search) ){
                $__search = wp_get_post_categories($post->ID);
                $search_string = 'category__in';
            }
        }

        if ( count($__search) ) {
            //search query
            if($is_work){
                $r = new WP_Query(
                    array(
                        'post_type' => A13_CUSTOM_POST_TYPE_WORK,
                        'tax_query' => array(
                            array(
                                'taxonomy' => A13_CPT_WORK_TAXONOMY,
                                'field' => 'slug',
                                'terms' => $__search,
                                'operator' => 'IN'
                            )
                        ),
                        'post__not_in' => array($post->ID),
                        'posts_per_page' => 3,
                        'no_found_rows' => true,
                        'post_status' => 'publish',
                        'ignore_sticky_posts' => true)
                );
            }
            else{
                $r = new WP_Query(
                    array(
                        $search_string => $__search,
                        'post__not_in' => array($post->ID),
                        'posts_per_page' => 3,
                        'no_found_rows' => true,
                        'post_status' => 'publish',
                        'ignore_sticky_posts' => true)
                );
            }

            if ($r->have_posts()) :
                echo '
                            <div class="in_post_widget'.($is_work? ' bricks_small non-fluid ' : '').'">
                                <h3 class="title widget-title"><span>'.$widget_title.'</span></h3>
                                <div class="widget-inside clearfix">';
                $background_classes = '';
                while ($r->have_posts()) : $r->the_post();
                    if($is_work){
                        
                        
                        $post_id            = get_the_ID();
                        $background_color   = get_post_meta($post_id, '_overlay_bg_color', true);
                        $url                = get_permalink();
                        $is_protected       = post_password_required($post_id) ? 1 : 0;
                        $image_size         = 'apollo-post-brick' ;
                        $post_thumbnail     = wpb_getImageBySize(array( 'post_id' => $post_id, 'thumb_size' => $image_size ));

                        $thumbnail_addr     = $post_thumbnail['p_img_large'][0];
                        $post_title         = get_the_title($post_id);
                        
                        $background_class   = "g-link-bgimg_".$post_id;
                        $background_classes .= ".".$background_class."{background-image:url('".$thumbnail_addr."');}";  
                        $categories_names   = '';
                        
                        $post_categories = wp_get_object_terms($post_id, 'genre' );
                        if(!is_wp_error($post_categories)) {
                            foreach($post_categories as $cat) {
                                $categories_names .= ' '.$cat->name.',';
                            }
                        }
                        $categories_names = substr($categories_names,0,-1);
                        
                        $teasers = '';

                        $teasers .= '<div class="g-item ready w1 h1">';
                        $teasers .= '<div data-post_id="'.esc_attr($post_id).'" class="g-link under" data-post-type="work" data-href="'.esc_url($url).'" id="' . esc_attr('album-'.$post_id) . '" data-pass-protection="'.esc_attr($is_protected).'">';
                        $teasers .= '<i class="'.esc_attr($background_class).'"></i>';
                        
                        
                        $teasers .= '<div class="overlay" style="background-color:'.$background_color.';">';//overlay
                        //$teasers .= ( $overlay_image != '' ? '<img alt="'.__('overlay_image', 'a13_shortcodes').'" src="'.esc_url($overlay_image).'">' :'');
                        $teasers .= '</div>';//overlay end
                        
                        $teasers .= '<div class="cov works" >';
                        
                        $teasers .= '<span><small>'.$categories_names.'</small><strong>'.$post_title.'</strong></span>';

                        $teasers .= $is_protected ? '<em class="filling_table pass_form"><span><span class="error"></span><div class="clearfix"></div><span>'.__('Enter password', 'a13_shortcodes').'</span><input type="password" class="post_password" name="post_password" data-post-id="'.esc_attr($post_id).'"><input type="button" class="submit_password" value="'.__('Unlock', 'a13_shortcodes').'"></span></em>' : '';
                        $teasers .= '</div>';
                        
                        
                        
                        
                        
                        $teasers .= '</div>';
      
                        $teasers .= '</div>';
                        
                        echo $teasers;
                    }
                    else{
                        $page_title = get_the_title();
                        $class = ''; //empty for easily commenting out
                        $image_size = 'apollo-post-brick' ;
                        $img = a13_make_post_image( get_the_ID(), $image_size );

                        echo '<div class="'.esc_attr('item'.$class).'">';

                        if(strlen($img)){
                            echo '
                                        <div class="item-image post-media">
                                            <a href="' . get_permalink() . '" title="' . esc_attr($page_title) . '">' . $img . '<em></em></a>
                                        </div>';
                        }

                        echo '<a class="post-title" href="' . esc_url(get_permalink()) . '" title="' . esc_attr($page_title) . '">' . $page_title . '</a>';

                        echo '</div>';
                    }

                endwhile;

                echo '</div></div>';
                if( strlen($background_classes) ){
                    echo "<style>".$background_classes."</style>";
                }
                // Reset the global $the_post as this query will have stomped on it
                wp_reset_postdata();

            endif;
        }
    }
}

// check the current post for the existence of a short code
if ( ! function_exists( 'a13_has_shortcode' ) ) :
function a13_has_shortcode($shortcode = '', $check_tile_count = false ) {
     
    $post_to_check = get_post(get_the_ID());

    // false because we have to search through the post content first
    $found = array(0,0);
     
    // if no short code was provided, return false
    if (!$shortcode or !$post_to_check) {
        return $found;
    }
     
    // return our final results
    return array(substr_count( $post_to_check->post_content, '[' . $shortcode ),($check_tile_count ? substr_count( $post_to_check->post_content, 'tile_mode="1"' ) : 0 ) );
}
endif;
